//
// Robot Factory Portfolio Expert Advisor
//
// Created with: Forex Robot Factory | Expert Advisor Generator
// Website: https://www.forexrobotacademy.com/expert-advisor-generator/
//
// Copyright 2020, Forex Software Ltd.
//
// This Portfolio Expert works in MetaTrader 4.
// It opens separate positions for each strategy.
// Every position has an unique magic number,
// which corresponds to the index of the strategy.
//

#property copyright "Forex Software Ltd."
#property version   "2.4"
#property strict

static input double Entry_Amount      = 0.01; // Entry lots
static input int    Base_Magic_Number = 100;  // Base Magic Number

#define TRADE_RETRY_COUNT 4
#define TRADE_RETRY_WAIT  100
#define OP_FLAT           -1

// Session time is set in seconds from 00:00
const int sessionSundayOpen           = 0;     // 00:00
const int sessionSundayClose          = 86400; // 24:00
const int sessionMondayThursdayOpen   = 0;     // 00:00
const int sessionMondayThursdayClose  = 86400; // 24:00
const int sessionFridayOpen           = 0;     // 00:00
const int sessionFridayClose          = 86400; // 24:00
const bool sessionIgnoreSunday        = false;
const bool sessionCloseAtSessionClose = false;
const bool sessionCloseAtFridayClose  = true;

const int    strategiesCount = 46;
const double sigma           = 0.000001;

datetime barTime;
int      digits;
double   stopLevel;
double   pip;
bool     setProtectionSeparately = false;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderScope
  {
   ORDER_SCOPE_UNDEFINED,
   ORDER_SCOPE_ENTRY,
   ORDER_SCOPE_EXIT
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderDirection
  {
   ORDER_DIRECTION_NONE,
   ORDER_DIRECTION_BUY,
   ORDER_DIRECTION_SELL,
   ORDER_DIRECTION_BOTH
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Position
  {
   int               Type;
   int               Ticket;
   int               MagicNumber;
   double            Lots;
   double            Price;
   double            StopLoss;
   double            TakeProfit;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Signal
  {
   int               MagicNumber;
   OrderScope        Scope;
   OrderDirection    Direction;
   int               StopLossPips;
   int               TakeProfitPips;
   bool              IsTrailingStop;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
  {
   barTime   = Time[0];
   digits    = (int) MarketInfo(_Symbol, MODE_DIGITS);
   stopLevel = MarketInfo(_Symbol, MODE_STOPLEVEL);
   pip       = GetPipValue();

   const ENUM_INIT_RETCODE initRetcode = ValidateInit();

   return (initRetcode);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(IsForceSessionClose())
     {
      CloseAllPositions();
      return;
     }

   if(Time[0]>barTime)
     {
      barTime=Time[0];
      OnBar();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnBar()
  {
   if(IsOutOfSession()) return;

   Signal signalList[];
   SetSignals(signalList);
   int signalsCount=ArraySize(signalList);

   for(int i=0;i<signalsCount;i++)
     {
      Signal signal=signalList[i];
      ManageSignal(signal);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ManageSignal(Signal &signal)
  {
   Position position=CreatePosition(signal.MagicNumber);

   if(position.Type!=OP_FLAT && signal.Scope==ORDER_SCOPE_EXIT)
     {
      if(signal.Direction==ORDER_DIRECTION_BOTH ||
         (position.Type==OP_BUY && signal.Direction==ORDER_DIRECTION_SELL) ||
         (position.Type==OP_SELL && signal.Direction==ORDER_DIRECTION_BUY))
        {
         ClosePosition(position);
        }
     }

   if(position.Type!=OP_FLAT && signal.Scope==ORDER_SCOPE_EXIT && signal.IsTrailingStop)
     {
      double trailingStop=GetTrailingStop(position,signal.StopLossPips);
      Print(trailingStop);
      ManageTrailingStop(position,trailingStop);
     }

   if(position.Type==OP_FLAT && signal.Scope==ORDER_SCOPE_ENTRY)
     {
      if(signal.Direction==ORDER_DIRECTION_BUY || signal.Direction==ORDER_DIRECTION_SELL)
        {
         OpenPosition(signal);
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Position CreatePosition(const int magicNumber)
  {
   Position position;
   position.MagicNumber = magicNumber;
   position.Type        = OP_FLAT;
   position.Ticket      = 0;
   position.Lots        = 0;
   position.Price       = 0;
   position.StopLoss    = 0;
   position.TakeProfit  = 0;

   int total=OrdersTotal();
   for(int pos=total-1; pos>=0; pos--)
     {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES) &&
         OrderSymbol()      == _Symbol &&
         OrderMagicNumber() == magicNumber &&
         OrderCloseTime()   == 0)
        {
         position.Type       = OrderType();
         position.Lots       = OrderLots();
         position.Ticket     = OrderTicket();
         position.Price      = OrderOpenPrice();
         position.StopLoss   = OrderStopLoss();
         position.TakeProfit = OrderTakeProfit();
         break;
        }
     }

   return (position);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal CreateEntrySignal(const int strategyIndex,const bool canOpenLong,const bool canOpenShort,const int stopLossPips,const int takeProfitPips, const bool isTrailingStop)
  {
   Signal signal;
   signal.MagicNumber    = GetMagicNumber(strategyIndex);
   signal.Scope          = ORDER_SCOPE_ENTRY;
   signal.Direction      = canOpenLong&&canOpenShort ? ORDER_DIRECTION_BOTH : canOpenLong ? ORDER_DIRECTION_BUY : canOpenShort ? ORDER_DIRECTION_SELL : ORDER_DIRECTION_NONE;
   signal.StopLossPips   = stopLossPips;
   signal.TakeProfitPips = takeProfitPips;
   signal.IsTrailingStop = isTrailingStop;

   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal CreateExitSignal(const int strategyIndex,const bool canCloseLong,const bool canCloseShorts,const int stopLossPips,const int takeProfitPips, const bool isTrailingStop)
  {
   Signal signal;
   signal.MagicNumber    = GetMagicNumber(strategyIndex);
   signal.Scope          = ORDER_SCOPE_EXIT;
   signal.Direction      = canCloseLong&&canCloseShorts ? ORDER_DIRECTION_BOTH : canCloseLong ? ORDER_DIRECTION_SELL : canCloseShorts ? ORDER_DIRECTION_BUY : ORDER_DIRECTION_NONE;
   signal.StopLossPips   = stopLossPips;
   signal.TakeProfitPips = takeProfitPips;
   signal.IsTrailingStop = isTrailingStop;

   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OpenPosition(Signal &signal)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      int    ticket     = 0;
      int    lastError  = 0;
      bool   modified   = false;
      int    command    = OrderDirectionToCommand(signal.Direction);
      double amount     = Entry_Amount;
      double stopLoss   = signal.StopLossPips>0 ? GetStopLoss(command, signal.StopLossPips) : 0;
      double takeProfit = signal.TakeProfitPips>0 ? GetTakeProfit(command, signal.TakeProfitPips) : 0;
      string comment    = IntegerToString(signal.MagicNumber);
      color  arrowColor = command==OP_BUY ? clrGreen : clrRed;

      if(IsTradeContextFree())
        {
         double price=MarketInfo(_Symbol,command==OP_BUY ? MODE_ASK : MODE_BID);
         if(setProtectionSeparately)
           {
            ticket=OrderSend(_Symbol,command,amount,price,10,0,0,comment,signal.MagicNumber,0,arrowColor);
            if(ticket>0 && (signal.StopLossPips>0 || signal.TakeProfitPips>0))
               modified=OrderModify(ticket,0,stopLoss,takeProfit,0,clrBlue);
           }
         else
           {
            ticket=OrderSend(_Symbol,command,amount,price,10,stopLoss,takeProfit,comment,signal.MagicNumber,0,arrowColor);
            lastError=GetLastError();
            if(ticket<=0 && lastError==130)
              {
               ticket=OrderSend(_Symbol,command,amount,price,10,0,0,comment,signal.MagicNumber,0,arrowColor);
               if(ticket>0 && (signal.StopLossPips>0 || signal.TakeProfitPips>0))
                  modified=OrderModify(ticket,0,stopLoss,takeProfit,0,clrBlue);
               if(ticket>0 && modified)
                 {
                  setProtectionSeparately=true;
                  Print("Detected ECN type position protection.");
                 }
              }
           }
        }

      if(ticket>0) break;

      lastError=GetLastError();

      if(lastError!=135 && lastError!=136 && lastError!=137 && lastError!=138) break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Open Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ClosePosition(Position &position)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      bool closed;
      int lastError=0;

      if(IsTradeContextFree())
        {
         double price=MarketInfo(_Symbol,position.Type==OP_BUY ? MODE_BID : MODE_ASK);
         closed=OrderClose(position.Ticket,position.Lots,price,10,clrYellow);
         lastError=GetLastError();
        }

      if(closed)
        {
         position.Type       = OP_FLAT;
         position.Lots       = 0;
         position.Price      = 0;
         position.StopLoss   = 0;
         position.TakeProfit = 0;

         break;
        }

      if(lastError==4108) break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Close Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ModifyPosition(Position &position)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      bool modified;
      int lastError=0;

      if(IsTradeContextFree())
        {
         modified=OrderModify(position.Ticket,0,position.StopLoss,position.TakeProfit,0,clrBlue);
         lastError=GetLastError();
        }

      if(modified)
        {
         const int magicNumber=position.MagicNumber;
         position=CreatePosition(magicNumber);

         break;
        }

      if(lastError==4108)
         break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Modify Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CloseAllPositions()
  {
   for(int i=0;i<strategiesCount;i++)
     {
      int magicNumber=GetMagicNumber(i);
      Position position=CreatePosition(magicNumber);

      if(position.Type==OP_BUY || position.Type==OP_SELL)
         ClosePosition(position);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetStopLoss(int command,int stopLossPips)
  {
   if(stopLossPips==0) return (0);

   double deltaPips      = MathMax(pip*stopLossPips, _Point*stopLevel);
   double referencePrice = MarketInfo(_Symbol, command==OP_BUY ? MODE_BID : MODE_ASK);
   double stopLossPoints = command==OP_BUY ? referencePrice-deltaPips : referencePrice+deltaPips;
   double finalStopLoss  = NormalizeDouble(stopLossPoints, digits);

   return (finalStopLoss);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetTakeProfit(int command,int takeProfitPips)
  {
   if(takeProfitPips==0) return (0);

   double deltaPips        = MathMax(pip*takeProfitPips, _Point*stopLevel);
   double referencePrice   = MarketInfo(_Symbol, command==OP_BUY ? MODE_BID : MODE_ASK);
   double takeProfitPoints = command==OP_BUY ? referencePrice+deltaPips : referencePrice-deltaPips;
   double finalTakeProfit  = NormalizeDouble(takeProfitPoints, digits);

   return (finalTakeProfit);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetTrailingStop(Position &position, int stopLoss)
  {
   double bid=MarketInfo(_Symbol,MODE_BID);
   double ask=MarketInfo(_Symbol,MODE_ASK);
   double stopLevelPoints=_Point*stopLevel;
   double stopLossPoints=pip*stopLoss;

   if(position.Type==OP_BUY)
     {
      double stopLossPrice=High[1]-stopLossPoints;
      if(position.StopLoss<stopLossPrice-pip)
        {
         if(stopLossPrice<bid)
           {
            if(stopLossPrice>=bid-stopLevelPoints)
               return (bid - stopLevelPoints);
            else
               return (stopLossPrice);
           }
         else
           {
            return (bid);
           }
        }
     }

   else if(position.Type==OP_SELL)
     {
      double stopLossPrice=Low[1]+stopLossPoints;
      if(position.StopLoss>stopLossPrice+pip)
        {
         if(stopLossPrice>ask)
           {
            if(stopLossPrice<=ask+stopLevelPoints)
               return (ask + stopLevelPoints);
            else
               return (stopLossPrice);
           }
         else
           {
            return (ask);
           }
        }
     }

   return (position.StopLoss);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ManageTrailingStop(Position &position, double trailingStop)
  {
   double bid=MarketInfo(_Symbol,MODE_BID);
   double ask=MarketInfo(_Symbol,MODE_ASK);

   if(position.Type==OP_BUY && MathAbs(trailingStop-bid)<_Point)
     {
      ClosePosition(position);
     }

   else if(position.Type==OP_SELL && MathAbs(trailingStop-ask)<_Point)
     {
      ClosePosition(position);
     }

   else if(MathAbs(trailingStop-position.StopLoss)>_Point)
     {
      position.StopLoss=NormalizeDouble(trailingStop,digits);
      ModifyPosition(position);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsTradeContextFree()
  {
   if(IsTradeAllowed()) return (true);

   uint startWait=GetTickCount();
   Print("Trade context is busy! Waiting...");

   while(true)
     {
      if(IsStopped()) return (false);

      uint diff=GetTickCount()-startWait;
      if(diff>30*1000)
        {
         Print("The waiting limit exceeded!");
         return (false);
        }

      if(IsTradeAllowed())
        {
         RefreshRates();
         return (true);
        }
      Sleep(TRADE_RETRY_WAIT);
     }

   return (true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsOutOfSession()
  {
   MqlDateTime time0; TimeToStruct(Time[0],time0);
   int weekDay           = time0.day_of_week;
   long timeFromMidnight = Time[0]%86400;
   int periodLength      = PeriodSeconds(_Period);
   bool skipTrade        = false;

   if(weekDay==0)
     {
      if(sessionIgnoreSunday) return true;
      int lastBarFix=sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionSundayOpen || timeFromMidnight+lastBarFix>sessionSundayClose;
     }
   else if(weekDay<5)
     {
      int lastBarFix=sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionMondayThursdayOpen || timeFromMidnight+lastBarFix>sessionMondayThursdayClose;
     }
   else
     {
      int lastBarFix=sessionCloseAtFridayClose || sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionFridayOpen || timeFromMidnight+lastBarFix>sessionFridayClose;
     }

   return (skipTrade);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsForceSessionClose()
  {
   if(!sessionCloseAtFridayClose && !sessionCloseAtSessionClose)
      return (false);

   MqlDateTime time0; TimeToStruct(Time[0],time0);
   int weekDay           = time0.day_of_week;
   long timeFromMidnight = Time[0]%86400;
   int periodLength      = PeriodSeconds(_Period);

   bool forceExit=false;
   if(weekDay==0 && sessionCloseAtSessionClose)
     {
      forceExit=timeFromMidnight+periodLength>sessionSundayClose;
     }
   else if(weekDay<5 && sessionCloseAtSessionClose)
     {
      forceExit=timeFromMidnight+periodLength>sessionMondayThursdayClose;
     }
   else if(weekDay==5)
     {
      forceExit=timeFromMidnight+periodLength>sessionFridayClose;
     }

   return (forceExit);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
datetime Time(int bar)
  {
   return Time[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Open(int bar)
  {
   return Open[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double High(int bar)
  {
   return High[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Low(int bar)
  {
   return Low[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Close(int bar)
  {
   return Close[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetPipValue()
  {
   switch((int) MarketInfo(_Symbol,MODE_DIGITS))
     {
      case  5: case  4: return (0.0001);
      case  3: case  2: return (0.01);
      case  1:          return (0.1);
     }

   return (1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetMagicNumber(int strategyIndex)
  {
   int magic=1000*Base_Magic_Number+strategyIndex;
   return (magic);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OrderDirectionToCommand(OrderDirection dir)
  {
   if(dir==ORDER_DIRECTION_BUY)  return (OP_BUY);
   if(dir==ORDER_DIRECTION_SELL) return (OP_SELL);

   return (OP_FLAT);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void SetSignals(Signal &signalList[])
  {
   int i=0;
   ArrayResize(signalList,2*strategiesCount);
   HideTestIndicators(true);

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":65,"takeProfit":72,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Commodity Channel Index","listIndexes":[4,5,0,0,0],"numValues":[6,-4,0,0,0,0]},{"name":"Moving Average","listIndexes":[1,0,3,0,0],"numValues":[11,0,0,0,0,0]},{"name":"Moving Average","listIndexes":[6,0,3,0,0],"numValues":[47,0,0,0,0,0]}],"closeFilters":[{"name":"Standard Deviation","listIndexes":[2,3,0,0,0],"numValues":[24,0.0017,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_00();
   signalList[i++] = GetEntrySignal_00();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":38,"takeProfit":55,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[74,3.67,0,0,0,0]}],"closeFilters":[{"name":"Alligator","listIndexes":[7,3,4,0,0],"numValues":[42,19,22,14,4,1]}]} */
   signalList[i++] = GetExitSignal_01();
   signalList[i++] = GetEntrySignal_01();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":54,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[31,1.7,0,0,0,0]},{"name":"Moving Average","listIndexes":[1,0,3,0,0],"numValues":[79,0,0,0,0,0]},{"name":"Stochastic Signal","listIndexes":[0,0,0,0,0],"numValues":[9,7,7,0,0,0]}],"closeFilters":[{"name":"Moving Average","listIndexes":[7,0,3,0,0],"numValues":[36,1,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_02();
   signalList[i++] = GetEntrySignal_02();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":46,"takeProfit":45,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Awesome Oscillator","listIndexes":[4,0,0,0,0],"numValues":[0,0,0,0,0,0]},{"name":"Stochastic Signal","listIndexes":[3,0,0,0,0],"numValues":[8,10,3,0,0,0]},{"name":"Volumes","listIndexes":[7,0,0,0,0],"numValues":[1000,0,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[0,3,0,0,0],"numValues":[33,2.56,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_03();
   signalList[i++] = GetEntrySignal_03();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":20,"takeProfit":14,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Force Index","listIndexes":[5,0,0,0,0],"numValues":[7,0,0,0,0,0]},{"name":"Stochastic","listIndexes":[4,0,0,0,0],"numValues":[18,17,1,25,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[3,3,0,0,0],"numValues":[53,3.35,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_04();
   signalList[i++] = GetEntrySignal_04();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":59,"takeProfit":101,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Alligator","listIndexes":[7,3,4,0,0],"numValues":[39,14,21,14,9,5]}],"closeFilters":[{"name":"Average True Range","listIndexes":[5,0,0,0,0],"numValues":[36,0.0015,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_05();
   signalList[i++] = GetEntrySignal_05();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":74,"takeProfit":51,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Commodity Channel Index","listIndexes":[4,5,0,0,0],"numValues":[7,-95,0,0,0,0]},{"name":"Volumes","listIndexes":[5,0,0,0,0],"numValues":[68,0,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[0,3,0,0,0],"numValues":[67,3.38,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_06();
   signalList[i++] = GetEntrySignal_06();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":49,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":true},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[19,43,0,0,0,0]},{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[40,2.49,0,0,0,0]}],"closeFilters":[{"name":"Williams' Percent Range","listIndexes":[5,0,0,0,0],"numValues":[44,-37,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_07();
   signalList[i++] = GetEntrySignal_07();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":54,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":true},"openFilters":[{"name":"Alligator","listIndexes":[9,3,4,0,0],"numValues":[44,20,24,11,47,2]}],"closeFilters":[{"name":"Momentum","listIndexes":[3,3,0,0,0],"numValues":[31,-24.86,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_08();
   signalList[i++] = GetEntrySignal_08();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":50,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":true},"openFilters":[{"name":"Force Index","listIndexes":[5,0,0,0,0],"numValues":[52,0,0,0,0,0]},{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[21,1.18,0,0,0,0]},{"name":"RVI Signal","listIndexes":[2,0,0,0,0],"numValues":[36,0,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[4,3,0,0,0],"numValues":[5,99,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_09();
   signalList[i++] = GetEntrySignal_09();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":57,"takeProfit":8,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Force Index","listIndexes":[5,0,0,0,0],"numValues":[47,0,0,0,0,0]},{"name":"Commodity Channel Index","listIndexes":[7,5,0,0,0],"numValues":[10,0,0,0,0,0]},{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[21,2.43,0,0,0,0]}],"closeFilters":[{"name":"Moving Average","listIndexes":[0,0,3,0,0],"numValues":[34,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_010();
   signalList[i++] = GetEntrySignal_010();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":84,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Money Flow Index","listIndexes":[4,0,0,0,0],"numValues":[72,56,0,0,0,0]},{"name":"Force Index","listIndexes":[3,0,0,0,0],"numValues":[21,0,0,0,0,0]}],"closeFilters":[{"name":"Average True Range","listIndexes":[4,0,0,0,0],"numValues":[23,0.0004,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_011();
   signalList[i++] = GetEntrySignal_011();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":80,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Bollinger Bands","listIndexes":[5,3,0,0,0],"numValues":[30,2.52,0,0,0,0]},{"name":"DeMarker","listIndexes":[1,0,0,0,0],"numValues":[7,0,0,0,0,0]},{"name":"Alligator","listIndexes":[0,3,4,0,0],"numValues":[15,14,14,9,9,5]}],"closeFilters":[{"name":"DeMarker","listIndexes":[4,0,0,0,0],"numValues":[27,0.67,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_012();
   signalList[i++] = GetEntrySignal_012();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":57,"takeProfit":68,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"RVI","listIndexes":[4,0,0,0,0],"numValues":[9,-0.01,0,0,0,0]},{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[16,28,0,0,0,0]}],"closeFilters":[{"name":"DeMarker","listIndexes":[4,0,0,0,0],"numValues":[31,0.25,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_013();
   signalList[i++] = GetEntrySignal_013();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":56,"takeProfit":33,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Momentum","listIndexes":[2,3,0,0,0],"numValues":[7,100.0483,0,0,0,0]},{"name":"Money Flow Index","listIndexes":[5,0,0,0,0],"numValues":[10,86,0,0,0,0]}],"closeFilters":[{"name":"Stochastic Signal","listIndexes":[3,0,0,0,0],"numValues":[3,1,1,0,0,0]}]} */
   signalList[i++] = GetExitSignal_014();
   signalList[i++] = GetEntrySignal_014();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":19,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RVI","listIndexes":[4,0,0,0,0],"numValues":[47,0.14,0,0,0,0]}],"closeFilters":[{"name":"Stochastic","listIndexes":[4,0,0,0,0],"numValues":[9,5,4,96,0,0]}]} */
   signalList[i++] = GetExitSignal_015();
   signalList[i++] = GetEntrySignal_015();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":62,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[9,61,0,0,0,0]}],"closeFilters":[{"name":"Awesome Oscillator","listIndexes":[4,0,0,0,0],"numValues":[0.0021,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_016();
   signalList[i++] = GetEntrySignal_016();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":43,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[3,0,0,0,0],"numValues":[59,27,0,0,0,0]},{"name":"Force Index","listIndexes":[5,0,0,0,0],"numValues":[65,0,0,0,0,0]},{"name":"Alligator","listIndexes":[3,3,4,0,0],"numValues":[43,18,11,15,7,1]}],"closeFilters":[{"name":"Stochastic","listIndexes":[2,0,0,0,0],"numValues":[45,1,4,97,0,0]}]} */
   signalList[i++] = GetExitSignal_017();
   signalList[i++] = GetEntrySignal_017();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":85,"takeProfit":44,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Stochastic Signal","listIndexes":[0,0,0,0,0],"numValues":[11,4,10,0,0,0]},{"name":"Volumes","listIndexes":[6,0,0,0,0],"numValues":[1000,0,0,0,0,0]},{"name":"Stochastic","listIndexes":[6,0,0,0,0],"numValues":[13,1,2,20,0,0]}],"closeFilters":[{"name":"Standard Deviation","listIndexes":[4,3,0,0,0],"numValues":[68,0.0031,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_018();
   signalList[i++] = GetEntrySignal_018();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":23,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"DeMarker","listIndexes":[5,0,0,0,0],"numValues":[28,0.13,0,0,0,0]}],"closeFilters":[{"name":"Moving Average","listIndexes":[2,0,3,0,0],"numValues":[12,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_019();
   signalList[i++] = GetEntrySignal_019();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Volumes","listIndexes":[4,0,0,0,0],"numValues":[989,0,0,0,0,0]},{"name":"Directional Indicators","listIndexes":[3,0,0,0,0],"numValues":[12,0,0,0,0,0]},{"name":"Alligator","listIndexes":[4,3,4,0,0],"numValues":[5,11,7,5,5,4]}],"closeFilters":[{"name":"RSI","listIndexes":[4,3,0,0,0],"numValues":[28,55,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_020();
   signalList[i++] = GetEntrySignal_020();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":60,"takeProfit":90,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Average True Range","listIndexes":[3,0,0,0,0],"numValues":[26,0.0013,0,0,0,0]},{"name":"Stochastic Signal","listIndexes":[0,0,0,0,0],"numValues":[13,4,1,0,0,0]},{"name":"Money Flow Index","listIndexes":[6,0,0,0,0],"numValues":[21,20,0,0,0,0]}],"closeFilters":[{"name":"Envelopes","listIndexes":[1,3,0,0,0],"numValues":[2,0.5,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_021();
   signalList[i++] = GetEntrySignal_021();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"ADX","listIndexes":[2,0,0,0,0],"numValues":[36,30,0,0,0,0]},{"name":"Money Flow Index","listIndexes":[5,0,0,0,0],"numValues":[5,58,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[2,3,0,0,0],"numValues":[18,66,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_022();
   signalList[i++] = GetEntrySignal_022();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":27,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"DeMarker","listIndexes":[5,0,0,0,0],"numValues":[28,0.13,0,0,0,0]}],"closeFilters":[{"name":"RVI Signal","listIndexes":[0,0,0,0,0],"numValues":[78,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_023();
   signalList[i++] = GetEntrySignal_023();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":8,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Bollinger Bands","listIndexes":[4,3,0,0,0],"numValues":[13,3.23,0,0,0,0]}],"closeFilters":[{"name":"RVI Signal","listIndexes":[0,0,0,0,0],"numValues":[64,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_024();
   signalList[i++] = GetEntrySignal_024();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":17,"takeProfit":13,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"DeMarker","listIndexes":[5,0,0,0,0],"numValues":[29,0.13,0,0,0,0]}],"closeFilters":[{"name":"Average True Range","listIndexes":[1,0,0,0,0],"numValues":[20,0.01,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_025();
   signalList[i++] = GetEntrySignal_025();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":16,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Directional Indicators","listIndexes":[2,0,0,0,0],"numValues":[7,0,0,0,0,0]},{"name":"Moving Average","listIndexes":[4,0,3,0,0],"numValues":[23,0,0,0,0,0]},{"name":"Force Index","listIndexes":[1,0,0,0,0],"numValues":[7,0,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[3,3,0,0,0],"numValues":[47,19,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_026();
   signalList[i++] = GetEntrySignal_026();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":82,"takeProfit":50,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RSI","listIndexes":[5,3,0,0,0],"numValues":[20,67,0,0,0,0]}],"closeFilters":[{"name":"Volumes","listIndexes":[5,0,0,0,0],"numValues":[1872,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_027();
   signalList[i++] = GetEntrySignal_027();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":38,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Force Index","listIndexes":[4,0,0,0,0],"numValues":[14,0,0,0,0,0]},{"name":"Accelerator Oscillator","listIndexes":[5,0,0,0,0],"numValues":[0,0,0,0,0,0]},{"name":"Volumes","listIndexes":[0,0,0,0,0],"numValues":[1000,0,0,0,0,0]}],"closeFilters":[{"name":"Williams' Percent Range","listIndexes":[2,0,0,0,0],"numValues":[48,-20,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_028();
   signalList[i++] = GetEntrySignal_028();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":63,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"DeMarker","listIndexes":[7,0,0,0,0],"numValues":[67,0,0,0,0,0]},{"name":"Stochastic","listIndexes":[5,0,0,0,0],"numValues":[65,7,17,25,0,0]}],"closeFilters":[{"name":"Money Flow Index","listIndexes":[5,0,0,0,0],"numValues":[43,72,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_029();
   signalList[i++] = GetEntrySignal_029();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":57,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":true},"openFilters":[{"name":"Money Flow Index","listIndexes":[5,0,0,0,0],"numValues":[9,89,0,0,0,0]}],"closeFilters":[{"name":"Stochastic Signal","listIndexes":[3,0,0,0,0],"numValues":[3,1,1,0,0,0]}]} */
   signalList[i++] = GetExitSignal_030();
   signalList[i++] = GetEntrySignal_030();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":31,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[10,15,0,0,0,0]},{"name":"RVI","listIndexes":[6,0,0,0,0],"numValues":[32,0,0,0,0,0]},{"name":"Awesome Oscillator","listIndexes":[0,0,0,0,0],"numValues":[0,0,0,0,0,0]}],"closeFilters":[{"name":"DeMarker","listIndexes":[4,0,0,0,0],"numValues":[15,0.9,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_031();
   signalList[i++] = GetEntrySignal_031();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Stochastic Signal","listIndexes":[0,0,0,0,0],"numValues":[6,4,3,0,0,0]},{"name":"Stochastic Signal","listIndexes":[0,0,0,0,0],"numValues":[3,2,2,0,0,0]},{"name":"Commodity Channel Index","listIndexes":[1,5,0,0,0],"numValues":[33,0,0,0,0,0]}],"closeFilters":[{"name":"RVI","listIndexes":[5,0,0,0,0],"numValues":[36,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_032();
   signalList[i++] = GetEntrySignal_032();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Awesome Oscillator","listIndexes":[5,0,0,0,0],"numValues":[0,0,0,0,0,0]},{"name":"Standard Deviation","listIndexes":[7,3,0,0,0],"numValues":[5,0,0,0,0,0]},{"name":"Moving Averages Crossover","listIndexes":[2,0,0,0,0],"numValues":[20,23,0,0,0,0]}],"closeFilters":[{"name":"Money Flow Index","listIndexes":[4,0,0,0,0],"numValues":[28,65,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_033();
   signalList[i++] = GetEntrySignal_033();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Pin Bar","listIndexes":[0,0,0,0,0],"numValues":[9,42,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[4,3,0,0,0],"numValues":[12,20,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_034();
   signalList[i++] = GetEntrySignal_034();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[0,0,0,0,0],"numValues":[39,35,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[0,3,0,0,0],"numValues":[20,2.84,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_035();
   signalList[i++] = GetEntrySignal_035();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":62,"takeProfit":64,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RSI","listIndexes":[5,3,0,0,0],"numValues":[18,30,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[3,3,0,0,0],"numValues":[23,2.23,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_036();
   signalList[i++] = GetEntrySignal_036();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":59,"takeProfit":86,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":true},"openFilters":[{"name":"Accelerator Oscillator","listIndexes":[4,0,0,0,0],"numValues":[0,0,0,0,0,0]}],"closeFilters":[{"name":"Envelopes","listIndexes":[2,3,0,0,0],"numValues":[33,0.38,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_037();
   signalList[i++] = GetEntrySignal_037();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":86,"takeProfit":92,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Pin Bar","listIndexes":[0,0,0,0,0],"numValues":[8,15,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[5,3,0,0,0],"numValues":[31,3.72,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_038();
   signalList[i++] = GetEntrySignal_038();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":14,"takeProfit":0,"useStopLoss":true,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Envelopes","listIndexes":[4,3,0,0,0],"numValues":[22,0.09,0,0,0,0]},{"name":"Moving Average","listIndexes":[5,0,3,0,0],"numValues":[13,0,0,0,0,0]}],"closeFilters":[{"name":"Directional Indicators","listIndexes":[0,0,0,0,0],"numValues":[48,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_039();
   signalList[i++] = GetEntrySignal_039();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Force Index","listIndexes":[4,0,0,0,0],"numValues":[13,0,0,0,0,0]}],"closeFilters":[{"name":"Bollinger Bands","listIndexes":[3,3,0,0,0],"numValues":[21,3,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_040();
   signalList[i++] = GetEntrySignal_040();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":31,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Commodity Channel Index","listIndexes":[5,5,0,0,0],"numValues":[40,24,0,0,0,0]},{"name":"Directional Indicators","listIndexes":[1,0,0,0,0],"numValues":[27,0,0,0,0,0]},{"name":"Moving Average","listIndexes":[1,0,3,0,0],"numValues":[38,0,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[4,3,0,0,0],"numValues":[33,61,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_041();
   signalList[i++] = GetEntrySignal_041();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Money Flow Index","listIndexes":[5,0,0,0,0],"numValues":[5,72,0,0,0,0]}],"closeFilters":[{"name":"Awesome Oscillator","listIndexes":[4,0,0,0,0],"numValues":[-0.0012,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_042();
   signalList[i++] = GetEntrySignal_042();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Envelopes","listIndexes":[4,3,0,0,0],"numValues":[28,0.22,0,0,0,0]}],"closeFilters":[{"name":"Momentum","listIndexes":[0,3,0,0,0],"numValues":[68,100,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_043();
   signalList[i++] = GetEntrySignal_043();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":23,"useStopLoss":false,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Awesome Oscillator","listIndexes":[5,0,0,0,0],"numValues":[0,0,0,0,0,0]}],"closeFilters":[{"name":"Stochastic","listIndexes":[4,0,0,0,0],"numValues":[8,1,8,92,0,0]}]} */
   signalList[i++] = GetExitSignal_044();
   signalList[i++] = GetEntrySignal_044();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":0,"takeProfit":0,"useStopLoss":false,"useTakeProfit":false,"isTrailingStop":false},"openFilters":[{"name":"Pin Bar","listIndexes":[0,0,0,0,0],"numValues":[6,17,0,0,0,0]}],"closeFilters":[{"name":"Envelopes","listIndexes":[1,3,0,0,0],"numValues":[27,0.12,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_045();
   signalList[i++] = GetEntrySignal_045();

   HideTestIndicators(false);
   if(i!=2*strategiesCount)
      ArrayResize(signalList,i);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_00()
  {
   double ind0val1 = iCCI(NULL,0,6,PRICE_TYPICAL,1);
   double ind0val2 = iCCI(NULL,0,6,PRICE_TYPICAL,2);
   bool ind0long  = ind0val1 > -4 + sigma && ind0val2 < -4 - sigma;
   bool ind0short = ind0val1 < -1*-4 - sigma && ind0val2 > -1*-4 + sigma;
   double ind1val1 = iMA(NULL,0,11,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,11,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma;
   double ind2val1 = iMA(NULL,0,47,0,MODE_SMA,PRICE_CLOSE,1);
   double ind2val2 = iMA(NULL,0,47,0,MODE_SMA,PRICE_CLOSE,2);
   double ind2val3 = iMA(NULL,0,47,0,MODE_SMA,PRICE_CLOSE,3);
   bool ind2long  = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;

   Signal signal = CreateEntrySignal(0, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 65, 72, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_00()
  {
   double ind3val1 = iStdDev(NULL,0,24,0,MODE_SMA,PRICE_CLOSE,1);
   bool ind3long  = ind3val1 > 0.0017 + sigma;
   bool ind3short = ind3long;

   Signal signal = CreateExitSignal(0, ind3long, ind3short, 65, 72, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_01()
  {
   double ind0upBand1 = iBands(NULL,0,74,3.67,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind0dnBand1 = iBands(NULL,0,74,3.67,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind0upBand2 = iBands(NULL,0,74,3.67,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind0dnBand2 = iBands(NULL,0,74,3.67,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind0long  = Open(0) < ind0dnBand1 - sigma && Open(1) > ind0dnBand2 + sigma;
   bool ind0short = Open(0) > ind0upBand1 + sigma && Open(1) < ind0upBand2 - sigma;

   Signal signal = CreateEntrySignal(1, ind0long, ind0short, 38, 55, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_01()
  {
   double ind1val1 = iAlligator(NULL,0,42,19,22,14,4,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,1);
   double ind1val2 = iAlligator(NULL,0,42,19,22,14,4,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,1);
   double ind1val3 = iAlligator(NULL,0,42,19,22,14,4,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,2);
   double ind1val4 = iAlligator(NULL,0,42,19,22,14,4,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;

   Signal signal = CreateExitSignal(1, ind1long, ind1short, 38, 55, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_02()
  {
   double ind0upBand1 = iBands(NULL,0,31,1.70,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind0dnBand1 = iBands(NULL,0,31,1.70,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind0upBand2 = iBands(NULL,0,31,1.70,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind0dnBand2 = iBands(NULL,0,31,1.70,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind0long  = Open(0) < ind0dnBand1 - sigma && Open(1) > ind0dnBand2 + sigma;
   bool ind0short = Open(0) > ind0upBand1 + sigma && Open(1) < ind0upBand2 - sigma;
   double ind1val1 = iMA(NULL,0,79,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,79,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma;
   double ind2val1 = iStochastic(NULL,0,9,7,7,MODE_SMA,0,MODE_MAIN,1);
   double ind2val2 = iStochastic(NULL,0,9,7,7,MODE_SMA,0,MODE_SIGNAL,1);
   double ind2val3 = iStochastic(NULL,0,9,7,7,MODE_SMA,0,MODE_MAIN,2);
   double ind2val4 = iStochastic(NULL,0,9,7,7,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind2long  = ind2val1 > ind2val2 + sigma && ind2val3 < ind2val4 - sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma && ind2val3 > ind2val4 + sigma;

   Signal signal = CreateEntrySignal(2, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 54, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_02()
  {
   double ind3val1 = iMA(NULL,0,36,1,MODE_SMA,PRICE_CLOSE,1);
   double ind3val2 = iMA(NULL,0,36,1,MODE_SMA,PRICE_CLOSE,2);
   double ind3val3 = iMA(NULL,0,36,1,MODE_SMA,PRICE_CLOSE,3);
   bool ind3long  = ind3val1 < ind3val2 - sigma && ind3val2 > ind3val3 + sigma;
   bool ind3short = ind3val1 > ind3val2 + sigma && ind3val2 < ind3val3 - sigma;

   Signal signal = CreateExitSignal(2, ind3long, ind3short, 54, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_03()
  {
   double ind0val1 = iAO(NULL,0,1);
   double ind0val2 = iAO(NULL,0,2);
   bool ind0long  = ind0val1 > 0.0000 + sigma && ind0val2 < 0.0000 - sigma;
   bool ind0short = ind0val1 < -1*0.0000 - sigma && ind0val2 > -1*0.0000 + sigma;
   double ind1val1 = iStochastic(NULL,0,8,10,3,MODE_SMA,0,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,8,10,3,MODE_SMA,0,MODE_SIGNAL,1);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma;
   double ind2val1 = (double) iVolume(NULL,0,1);
   double ind2val2 = (double) iVolume(NULL,0,2);
   double ind2val3 = (double) iVolume(NULL,0,3);
   bool ind2long  = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;
   bool ind2short = ind2long;

   Signal signal = CreateEntrySignal(3, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 46, 45, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_03()
  {
   double ind3upBand1 = iBands(NULL,0,33,2.56,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind3dnBand1 = iBands(NULL,0,33,2.56,0,PRICE_CLOSE,MODE_LOWER,1);
   bool ind3long  = Open(0) > ind3upBand1 + sigma;
   bool ind3short = Open(0) < ind3dnBand1 - sigma;

   Signal signal = CreateExitSignal(3, ind3long, ind3short, 46, 45, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_04()
  {
   double ind0val1 = iForce(NULL,0,7,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iForce(NULL,0,7,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 0 - sigma && ind0val2 > 0 + sigma;
   bool ind0short = ind0val1 > 0 + sigma && ind0val2 < 0 - sigma;
   double ind1val1 = iStochastic(NULL,0,18,17,1,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,18,17,1,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind1long  = ind1val1 > 25 + sigma && ind1val2 < 25 - sigma;
   bool ind1short = ind1val1 < 100 - 25 - sigma && ind1val2 > 100 - 25 + sigma;

   Signal signal = CreateEntrySignal(4, ind0long && ind1long, ind0short && ind1short, 20, 14, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_04()
  {
   double ind2upBand1 = iBands(NULL,0,53,3.35,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind2dnBand1 = iBands(NULL,0,53,3.35,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind2upBand2 = iBands(NULL,0,53,3.35,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind2dnBand2 = iBands(NULL,0,53,3.35,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind2long  = Open(0) > ind2upBand1 + sigma && Open(1) < ind2upBand2 - sigma;
   bool ind2short = Open(0) < ind2dnBand1 - sigma && Open(1) > ind2dnBand2 + sigma;

   Signal signal = CreateExitSignal(4, ind2long, ind2short, 20, 14, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_05()
  {
   double ind0val1 = iAlligator(NULL,0,39,14,21,14,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,1);
   double ind0val2 = iAlligator(NULL,0,39,14,21,14,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,1);
   double ind0val3 = iAlligator(NULL,0,39,14,21,14,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,2);
   double ind0val4 = iAlligator(NULL,0,39,14,21,14,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;

   Signal signal = CreateEntrySignal(5, ind0long, ind0short, 59, 101, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_05()
  {
   double ind1val1 = iATR(NULL,0,36,1);
   double ind1val2 = iATR(NULL,0,36,2);
   bool ind1long  = ind1val1 < 0.0015 - sigma && ind1val2 > 0.0015 + sigma;
   bool ind1short = ind1long;

   Signal signal = CreateExitSignal(5, ind1long, ind1short, 59, 101, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_06()
  {
   double ind0val1 = iCCI(NULL,0,7,PRICE_TYPICAL,1);
   double ind0val2 = iCCI(NULL,0,7,PRICE_TYPICAL,2);
   bool ind0long  = ind0val1 > -95 + sigma && ind0val2 < -95 - sigma;
   bool ind0short = ind0val1 < -1*-95 - sigma && ind0val2 > -1*-95 + sigma;
   double ind1val1 = (double) iVolume(NULL,0,1);
   double ind1val2 = (double) iVolume(NULL,0,2);
   bool ind1long  = ind1val1 < 68 - sigma && ind1val2 > 68 + sigma;
   bool ind1short = ind1long;

   Signal signal = CreateEntrySignal(6, ind0long && ind1long, ind0short && ind1short, 74, 51, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_06()
  {
   double ind2upBand1 = iBands(NULL,0,67,3.38,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind2dnBand1 = iBands(NULL,0,67,3.38,0,PRICE_CLOSE,MODE_LOWER,1);
   bool ind2long  = Open(0) > ind2upBand1 + sigma;
   bool ind2short = Open(0) < ind2dnBand1 - sigma;

   Signal signal = CreateExitSignal(6, ind2long, ind2short, 74, 51, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_07()
  {
   double ind0val1 = iMA(NULL,0,19,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,43,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,19,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,43,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   double ind1upBand1 = iBands(NULL,0,40,2.49,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,40,2.49,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind1upBand2 = iBands(NULL,0,40,2.49,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind1dnBand2 = iBands(NULL,0,40,2.49,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind1long  = Open(0) < ind1dnBand1 - sigma && Open(1) > ind1dnBand2 + sigma;
   bool ind1short = Open(0) > ind1upBand1 + sigma && Open(1) < ind1upBand2 - sigma;

   Signal signal = CreateEntrySignal(7, ind0long && ind1long, ind0short && ind1short, 49, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_07()
  {
   double ind2val1 = iWPR(NULL,0,44,1);
   double ind2val2 = iWPR(NULL,0,44,2);
   bool ind2long  = ind2val1 < -37 - sigma && ind2val2 > -37 + sigma;
   bool ind2short = ind2val1 > -100 - -37 + sigma && ind2val2 < -100 - -37 - sigma;

   Signal signal = CreateExitSignal(7, ind2long, ind2short, 49, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_08()
  {
   double ind0val1 = iAlligator(NULL,0,44,20,24,11,47,2,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,1);
   double ind0val2 = iAlligator(NULL,0,44,20,24,11,47,2,MODE_SMMA,PRICE_MEDIAN,MODE_GATORJAW,1);
   double ind0val3 = iAlligator(NULL,0,44,20,24,11,47,2,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,2);
   double ind0val4 = iAlligator(NULL,0,44,20,24,11,47,2,MODE_SMMA,PRICE_MEDIAN,MODE_GATORJAW,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;

   Signal signal = CreateEntrySignal(8, ind0long, ind0short, 54, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_08()
  {
   double ind1val1 = iMomentum(NULL,0,31,PRICE_CLOSE,1);
   bool ind1long  = ind1val1 < -24.8600 - sigma;
   bool ind1short = ind1val1 > 200 - -24.8600 + sigma;

   Signal signal = CreateExitSignal(8, ind1long, ind1short, 54, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_09()
  {
   double ind0val1 = iForce(NULL,0,52,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iForce(NULL,0,52,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 0 - sigma && ind0val2 > 0 + sigma;
   bool ind0short = ind0val1 > 0 + sigma && ind0val2 < 0 - sigma;
   double ind1upBand1 = iBands(NULL,0,21,1.18,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,21,1.18,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind1upBand2 = iBands(NULL,0,21,1.18,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind1dnBand2 = iBands(NULL,0,21,1.18,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind1long  = Open(0) < ind1dnBand1 - sigma && Open(1) > ind1dnBand2 + sigma;
   bool ind1short = Open(0) > ind1upBand1 + sigma && Open(1) < ind1upBand2 - sigma;
   double ind2val1 = iRVI(NULL,0,36,MODE_MAIN,1) - iRVI(NULL,0,36,MODE_SIGNAL,1);
   bool ind2long  = ind2val1 > 0 + sigma;
   bool ind2short = ind2val1 < 0 - sigma;

   Signal signal = CreateEntrySignal(9, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 50, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_09()
  {
   double ind3val1 = iRSI(NULL,0,5,PRICE_CLOSE,1);
   double ind3val2 = iRSI(NULL,0,5,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > 99 + sigma && ind3val2 < 99 - sigma;
   bool ind3short = ind3val1 < 100 - 99 - sigma && ind3val2 > 100 - 99 + sigma;

   Signal signal = CreateExitSignal(9, ind3long, ind3short, 50, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_010()
  {
   double ind0val1 = iForce(NULL,0,47,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iForce(NULL,0,47,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 0 - sigma && ind0val2 > 0 + sigma;
   bool ind0short = ind0val1 > 0 + sigma && ind0val2 < 0 - sigma;
   double ind1val1 = iCCI(NULL,0,10,PRICE_TYPICAL,1);
   double ind1val2 = iCCI(NULL,0,10,PRICE_TYPICAL,2);
   double ind1val3 = iCCI(NULL,0,10,PRICE_TYPICAL,3);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val2 > ind1val3 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val2 < ind1val3 - sigma;
   double ind2upBand1 = iBands(NULL,0,21,2.43,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind2dnBand1 = iBands(NULL,0,21,2.43,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind2upBand2 = iBands(NULL,0,21,2.43,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind2dnBand2 = iBands(NULL,0,21,2.43,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind2long  = Open(0) < ind2dnBand1 - sigma && Open(1) > ind2dnBand2 + sigma;
   bool ind2short = Open(0) > ind2upBand1 + sigma && Open(1) < ind2upBand2 - sigma;

   Signal signal = CreateEntrySignal(10, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 57, 8, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_010()
  {
   double ind3val1 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,1);
   double ind3val2 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > ind3val2 + sigma;
   bool ind3short = ind3val1 < ind3val2 - sigma;

   Signal signal = CreateExitSignal(10, ind3long, ind3short, 57, 8, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_011()
  {
   double ind0val1 = iMFI(NULL,0,72,1);
   double ind0val2 = iMFI(NULL,0,72,2);
   bool ind0long  = ind0val1 > 56 + sigma && ind0val2 < 56 - sigma;
   bool ind0short = ind0val1 < 100 - 56 - sigma && ind0val2 > 100 - 56 + sigma;
   double ind1val1 = iForce(NULL,0,21,MODE_SMA,PRICE_CLOSE,1);
   bool ind1long  = ind1val1 < 0 - sigma;
   bool ind1short = ind1val1 > 0 + sigma;

   Signal signal = CreateEntrySignal(11, ind0long && ind1long, ind0short && ind1short, 0, 84, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_011()
  {
   double ind2val1 = iATR(NULL,0,23,1);
   double ind2val2 = iATR(NULL,0,23,2);
   bool ind2long  = ind2val1 > 0.0004 + sigma && ind2val2 < 0.0004 - sigma;
   bool ind2short = ind2long;

   Signal signal = CreateExitSignal(11, ind2long, ind2short, 0, 84, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_012()
  {
   double ind0upBand1 = iBands(NULL,0,30,2.52,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind0dnBand1 = iBands(NULL,0,30,2.52,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind0upBand2 = iBands(NULL,0,30,2.52,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind0dnBand2 = iBands(NULL,0,30,2.52,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind0long  = Open(0) > ind0dnBand1 + sigma && Open(1) < ind0dnBand2 - sigma;
   bool ind0short = Open(0) < ind0upBand1 - sigma && Open(1) > ind0upBand2 + sigma;
   double ind1val1 = iDeMarker(NULL,0,7,1);
   double ind1val2 = iDeMarker(NULL,0,7,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma;
   double ind2val1 = iAlligator(NULL,0,15,14,14,9,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,1);
   double ind2val2 = iAlligator(NULL,0,15,14,14,9,9,5,MODE_SMMA,PRICE_MEDIAN,MODE_GATORLIPS,2);
   bool ind2long  = ind2val1 > ind2val2 + sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma;

   Signal signal = CreateEntrySignal(12, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 80, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_012()
  {
   double ind3val1 = iDeMarker(NULL,0,27,1);
   double ind3val2 = iDeMarker(NULL,0,27,2);
   bool ind3long  = ind3val1 > 0.67 + sigma && ind3val2 < 0.67 - sigma;
   bool ind3short = ind3val1 < 1 - 0.67 - sigma && ind3val2 > 1 - 0.67 + sigma;

   Signal signal = CreateExitSignal(12, ind3long, ind3short, 0, 80, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_013()
  {
   double ind0val1 = iRVI(NULL,0,9,MODE_MAIN,1);
   double ind0val2 = iRVI(NULL,0,9,MODE_MAIN,2);
   bool ind0long  = ind0val1 > -0.01 + sigma && ind0val2 < -0.01 - sigma;
   bool ind0short = ind0val1 < -1*-0.01 - sigma && ind0val2 > -1*-0.01 + sigma;
   double ind1val1 = iMA(NULL,0,16,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,28,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val3 = iMA(NULL,0,16,0,MODE_SMA,PRICE_CLOSE,2);
   double ind1val4 = iMA(NULL,0,28,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;

   Signal signal = CreateEntrySignal(13, ind0long && ind1long, ind0short && ind1short, 57, 68, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_013()
  {
   double ind2val1 = iDeMarker(NULL,0,31,1);
   double ind2val2 = iDeMarker(NULL,0,31,2);
   bool ind2long  = ind2val1 > 0.25 + sigma && ind2val2 < 0.25 - sigma;
   bool ind2short = ind2val1 < 1 - 0.25 - sigma && ind2val2 > 1 - 0.25 + sigma;

   Signal signal = CreateExitSignal(13, ind2long, ind2short, 57, 68, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_014()
  {
   double ind0val1 = iMomentum(NULL,0,7,PRICE_CLOSE,1);
   bool ind0long  = ind0val1 > 100.0483 + sigma;
   bool ind0short = ind0val1 < 200 - 100.0483 - sigma;
   double ind1val1 = iMFI(NULL,0,10,1);
   double ind1val2 = iMFI(NULL,0,10,2);
   bool ind1long  = ind1val1 < 86 - sigma && ind1val2 > 86 + sigma;
   bool ind1short = ind1val1 > 100 - 86 + sigma && ind1val2 < 100 - 86 - sigma;

   Signal signal = CreateEntrySignal(14, ind0long && ind1long, ind0short && ind1short, 56, 33, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_014()
  {
   double ind2val1 = iStochastic(NULL,0,3,1,1,MODE_SMA,0,MODE_MAIN,1);
   double ind2val2 = iStochastic(NULL,0,3,1,1,MODE_SMA,0,MODE_SIGNAL,1);
   bool ind2long  = ind2val1 < ind2val2 - sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma;

   Signal signal = CreateExitSignal(14, ind2long, ind2short, 56, 33, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_015()
  {
   double ind0val1 = iRVI(NULL,0,47,MODE_MAIN,1);
   double ind0val2 = iRVI(NULL,0,47,MODE_MAIN,2);
   bool ind0long  = ind0val1 > 0.14 + sigma && ind0val2 < 0.14 - sigma;
   bool ind0short = ind0val1 < -1*0.14 - sigma && ind0val2 > -1*0.14 + sigma;

   Signal signal = CreateEntrySignal(15, ind0long, ind0short, 0, 19, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_015()
  {
   double ind1val1 = iStochastic(NULL,0,9,5,4,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,9,5,4,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind1long  = ind1val1 > 96 + sigma && ind1val2 < 96 - sigma;
   bool ind1short = ind1val1 < 100 - 96 - sigma && ind1val2 > 100 - 96 + sigma;

   Signal signal = CreateExitSignal(15, ind1long, ind1short, 0, 19, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_016()
  {
   double ind0val1 = iMA(NULL,0,9,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,61,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,9,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,61,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;

   Signal signal = CreateEntrySignal(16, ind0long, ind0short, 0, 62, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_016()
  {
   double ind1val1 = iAO(NULL,0,1);
   double ind1val2 = iAO(NULL,0,2);
   bool ind1long  = ind1val1 > 0.0021 + sigma && ind1val2 < 0.0021 - sigma;
   bool ind1short = ind1val1 < -1*0.0021 - sigma && ind1val2 > -1*0.0021 + sigma;

   Signal signal = CreateExitSignal(16, ind1long, ind1short, 0, 62, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_017()
  {
   double ind0val1 = iMA(NULL,0,59,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,27,0,MODE_SMA,PRICE_CLOSE,1);
   bool ind0long  = ind0val1 < ind0val2 - sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma;
   double ind1val1 = iForce(NULL,0,65,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iForce(NULL,0,65,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 < 0 - sigma && ind1val2 > 0 + sigma;
   bool ind1short = ind1val1 > 0 + sigma && ind1val2 < 0 - sigma;
   double ind2val1 = iAlligator(NULL,0,43,18,11,15,7,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,1);
   double ind2val2 = iAlligator(NULL,0,43,18,11,15,7,1,MODE_SMMA,PRICE_MEDIAN,MODE_GATORTEETH,2);
   bool ind2long  = ind2val1 < ind2val2 - sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma;

   Signal signal = CreateEntrySignal(17, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 43, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_017()
  {
   double ind3val1 = iStochastic(NULL,0,45,1,4,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   bool ind3long  = ind3val1 > 97 + sigma;
   bool ind3short = ind3val1 < 100 - 97 - sigma;

   Signal signal = CreateExitSignal(17, ind3long, ind3short, 0, 43, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_018()
  {
   double ind0val1 = iStochastic(NULL,0,11,4,10,MODE_SMA,0,MODE_MAIN,1);
   double ind0val2 = iStochastic(NULL,0,11,4,10,MODE_SMA,0,MODE_SIGNAL,1);
   double ind0val3 = iStochastic(NULL,0,11,4,10,MODE_SMA,0,MODE_MAIN,2);
   double ind0val4 = iStochastic(NULL,0,11,4,10,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind0long  = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   bool ind0short = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   double ind1val1 = (double) iVolume(NULL,0,1);
   double ind1val2 = (double) iVolume(NULL,0,2);
   double ind1val3 = (double) iVolume(NULL,0,3);
   bool ind1long  = ind1val1 > ind1val2 + sigma && ind1val2 < ind1val3 - sigma;
   bool ind1short = ind1long;
   double ind2val1 = iStochastic(NULL,0,13,1,2,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind2val2 = iStochastic(NULL,0,13,1,2,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   double ind2val3 = iStochastic(NULL,0,13,1,2,MODE_SMA,STO_LOWHIGH,MODE_MAIN,3);
   bool ind2long  = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;

   Signal signal = CreateEntrySignal(18, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 85, 44, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_018()
  {
   double ind3val1 = iStdDev(NULL,0,68,0,MODE_SMA,PRICE_CLOSE,1);
   double ind3val2 = iStdDev(NULL,0,68,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > 0.0031 + sigma && ind3val2 < 0.0031 - sigma;
   bool ind3short = ind3long;

   Signal signal = CreateExitSignal(18, ind3long, ind3short, 85, 44, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_019()
  {
   double ind0val1 = iDeMarker(NULL,0,28,1);
   double ind0val2 = iDeMarker(NULL,0,28,2);
   bool ind0long  = ind0val1 < 0.13 - sigma && ind0val2 > 0.13 + sigma;
   bool ind0short = ind0val1 > 1 - 0.13 + sigma && ind0val2 < 1 - 0.13 - sigma;

   Signal signal = CreateEntrySignal(19, ind0long, ind0short, 23, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_019()
  {
   double ind1val1 = iMA(NULL,0,12,0,MODE_SMA,PRICE_CLOSE,1);
   bool ind1long  = Open(0) > ind1val1 + sigma;
   bool ind1short = Open(0) < ind1val1 - sigma;

   Signal signal = CreateExitSignal(19, ind1long, ind1short, 23, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_020()
  {
   double ind0val1 = (double) iVolume(NULL,0,1);
   double ind0val2 = (double) iVolume(NULL,0,2);
   bool ind0long  = ind0val1 > 989 + sigma && ind0val2 < 989 - sigma;
   bool ind0short = ind0long;
   double ind1val1 = iADX(NULL,0,12,PRICE_CLOSE,1,1) - iADX(NULL,0,12,PRICE_CLOSE,2,1);
   bool ind1long  = ind1val1 < 0 - sigma;
   bool ind1short = ind1val1 > 0 + sigma;
   double ind2val1 = iAlligator(NULL,0,5,11,7,5,5,4,MODE_SMMA,PRICE_MEDIAN,MODE_GATORJAW,1);
   double ind2val2 = iAlligator(NULL,0,5,11,7,5,5,4,MODE_SMMA,PRICE_MEDIAN,MODE_GATORJAW,2);
   bool ind2long  = ind2val1 > ind2val2 + sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma;

   Signal signal = CreateEntrySignal(20, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_020()
  {
   double ind3val1 = iRSI(NULL,0,28,PRICE_CLOSE,1);
   double ind3val2 = iRSI(NULL,0,28,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > 55 + sigma && ind3val2 < 55 - sigma;
   bool ind3short = ind3val1 < 100 - 55 - sigma && ind3val2 > 100 - 55 + sigma;

   Signal signal = CreateExitSignal(20, ind3long, ind3short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_021()
  {
   double ind0val1 = iATR(NULL,0,26,1);
   bool ind0long  = ind0val1 < 0.0013 - sigma;
   bool ind0short = ind0long;
   double ind1val1 = iStochastic(NULL,0,13,4,1,MODE_SMA,0,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,13,4,1,MODE_SMA,0,MODE_SIGNAL,1);
   double ind1val3 = iStochastic(NULL,0,13,4,1,MODE_SMA,0,MODE_MAIN,2);
   double ind1val4 = iStochastic(NULL,0,13,4,1,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind1long  = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;
   bool ind1short = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   double ind2val1 = iMFI(NULL,0,21,1);
   double ind2val2 = iMFI(NULL,0,21,2);
   double ind2val3 = iMFI(NULL,0,21,3);
   bool ind2long  = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;

   Signal signal = CreateEntrySignal(21, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 60, 90, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_021()
  {
   double ind3upBand1 = iEnvelopes(NULL,0,2,MODE_SMA,0,PRICE_CLOSE,0.50,MODE_UPPER,1);
   double ind3dnBand1 = iEnvelopes(NULL,0,2,MODE_SMA,0,PRICE_CLOSE,0.50,MODE_LOWER,1);
   bool ind3long  = Open(0) < ind3dnBand1 - sigma;
   bool ind3short = Open(0) > ind3upBand1 + sigma;

   Signal signal = CreateExitSignal(21, ind3long, ind3short, 60, 90, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_022()
  {
   double ind0val1 = iADX(NULL,0,36,PRICE_CLOSE,0,1);
   bool ind0long  = ind0val1 > 30 + sigma;
   bool ind0short = ind0long;
   double ind1val1 = iMFI(NULL,0,5,1);
   double ind1val2 = iMFI(NULL,0,5,2);
   bool ind1long  = ind1val1 < 58 - sigma && ind1val2 > 58 + sigma;
   bool ind1short = ind1val1 > 100 - 58 + sigma && ind1val2 < 100 - 58 - sigma;

   Signal signal = CreateEntrySignal(22, ind0long && ind1long, ind0short && ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_022()
  {
   double ind2val1 = iRSI(NULL,0,18,PRICE_CLOSE,1);
   bool ind2long  = ind2val1 > 66 + sigma;
   bool ind2short = ind2val1 < 100 - 66 - sigma;

   Signal signal = CreateExitSignal(22, ind2long, ind2short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_023()
  {
   double ind0val1 = iDeMarker(NULL,0,28,1);
   double ind0val2 = iDeMarker(NULL,0,28,2);
   bool ind0long  = ind0val1 < 0.13 - sigma && ind0val2 > 0.13 + sigma;
   bool ind0short = ind0val1 > 1 - 0.13 + sigma && ind0val2 < 1 - 0.13 - sigma;

   Signal signal = CreateEntrySignal(23, ind0long, ind0short, 27, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_023()
  {
   double ind1val1 = iRVI(NULL,0,78,MODE_MAIN,1) - iRVI(NULL,0,78,MODE_SIGNAL,1);
   double ind1val2 = iRVI(NULL,0,78,MODE_MAIN,2) - iRVI(NULL,0,78,MODE_SIGNAL,2);
   bool ind1long  = ind1val1 > 0 + sigma && ind1val2 < 0 - sigma;
   bool ind1short = ind1val1 < 0 - sigma && ind1val2 > 0 + sigma;

   Signal signal = CreateExitSignal(23, ind1long, ind1short, 27, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_024()
  {
   double ind0upBand1 = iBands(NULL,0,13,3.23,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind0dnBand1 = iBands(NULL,0,13,3.23,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind0upBand2 = iBands(NULL,0,13,3.23,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind0dnBand2 = iBands(NULL,0,13,3.23,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind0long  = Open(0) < ind0dnBand1 - sigma && Open(1) > ind0dnBand2 + sigma;
   bool ind0short = Open(0) > ind0upBand1 + sigma && Open(1) < ind0upBand2 - sigma;

   Signal signal = CreateEntrySignal(24, ind0long, ind0short, 0, 8, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_024()
  {
   double ind1val1 = iRVI(NULL,0,64,MODE_MAIN,1) - iRVI(NULL,0,64,MODE_SIGNAL,1);
   double ind1val2 = iRVI(NULL,0,64,MODE_MAIN,2) - iRVI(NULL,0,64,MODE_SIGNAL,2);
   bool ind1long  = ind1val1 > 0 + sigma && ind1val2 < 0 - sigma;
   bool ind1short = ind1val1 < 0 - sigma && ind1val2 > 0 + sigma;

   Signal signal = CreateExitSignal(24, ind1long, ind1short, 0, 8, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_025()
  {
   double ind0val1 = iDeMarker(NULL,0,29,1);
   double ind0val2 = iDeMarker(NULL,0,29,2);
   bool ind0long  = ind0val1 < 0.13 - sigma && ind0val2 > 0.13 + sigma;
   bool ind0short = ind0val1 > 1 - 0.13 + sigma && ind0val2 < 1 - 0.13 - sigma;

   Signal signal = CreateEntrySignal(25, ind0long, ind0short, 17, 13, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_025()
  {
   double ind1val1 = iATR(NULL,0,20,1);
   double ind1val2 = iATR(NULL,0,20,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1long;

   Signal signal = CreateExitSignal(25, ind1long, ind1short, 17, 13, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_026()
  {
   double ind0val1 = iADX(NULL,0,7,PRICE_CLOSE,1,1) - iADX(NULL,0,7,PRICE_CLOSE,2,1);
   bool ind0long  = ind0val1 > 0 + sigma;
   bool ind0short = ind0val1 < 0 - sigma;
   double ind1val1 = iMA(NULL,0,23,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,23,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = Open(0) > ind1val1 + sigma && Open(1) < ind1val2 - sigma;
   bool ind1short = Open(0) < ind1val1 - sigma && Open(1) > ind1val2 + sigma;
   double ind2val1 = iForce(NULL,0,7,MODE_SMA,PRICE_CLOSE,1);
   double ind2val2 = iForce(NULL,0,7,MODE_SMA,PRICE_CLOSE,2);
   bool ind2long  = ind2val1 < ind2val2 - sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma;

   Signal signal = CreateEntrySignal(26, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 16, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_026()
  {
   double ind3val1 = iRSI(NULL,0,47,PRICE_CLOSE,1);
   bool ind3long  = ind3val1 < 19 - sigma;
   bool ind3short = ind3val1 > 100 - 19 + sigma;

   Signal signal = CreateExitSignal(26, ind3long, ind3short, 0, 16, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_027()
  {
   double ind0val1 = iRSI(NULL,0,20,PRICE_CLOSE,1);
   double ind0val2 = iRSI(NULL,0,20,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 67 - sigma && ind0val2 > 67 + sigma;
   bool ind0short = ind0val1 > 100 - 67 + sigma && ind0val2 < 100 - 67 - sigma;

   Signal signal = CreateEntrySignal(27, ind0long, ind0short, 82, 50, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_027()
  {
   double ind1val1 = (double) iVolume(NULL,0,1);
   double ind1val2 = (double) iVolume(NULL,0,2);
   bool ind1long  = ind1val1 < 1872 - sigma && ind1val2 > 1872 + sigma;
   bool ind1short = ind1long;

   Signal signal = CreateExitSignal(27, ind1long, ind1short, 82, 50, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_028()
  {
   double ind0val1 = iForce(NULL,0,14,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iForce(NULL,0,14,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 > 0 + sigma && ind0val2 < 0 - sigma;
   bool ind0short = ind0val1 < 0 - sigma && ind0val2 > 0 + sigma;
   double ind1val1 = iAC(NULL,0,1);
   double ind1val2 = iAC(NULL,0,2);
   bool ind1long  = ind1val1 < 0.0000 - sigma && ind1val2 > 0.0000 + sigma;
   bool ind1short = ind1val1 > -1*0.0000 + sigma && ind1val2 < -1*0.0000 - sigma;
   double ind2val1 = (double) iVolume(NULL,0,1);
   double ind2val2 = (double) iVolume(NULL,0,2);
   bool ind2long  = ind2val1 > ind2val2 + sigma;
   bool ind2short = ind2long;

   Signal signal = CreateEntrySignal(28, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 38, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_028()
  {
   double ind3val1 = iWPR(NULL,0,48,1);
   bool ind3long  = ind3val1 > -20 + sigma;
   bool ind3short = ind3val1 < -100 - -20 - sigma;

   Signal signal = CreateExitSignal(28, ind3long, ind3short, 38, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_029()
  {
   double ind0val1 = iDeMarker(NULL,0,67,1);
   double ind0val2 = iDeMarker(NULL,0,67,2);
   double ind0val3 = iDeMarker(NULL,0,67,3);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val2 > ind0val3 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val2 < ind0val3 - sigma;
   double ind1val1 = iStochastic(NULL,0,65,7,17,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,65,7,17,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind1long  = ind1val1 < 25 - sigma && ind1val2 > 25 + sigma;
   bool ind1short = ind1val1 > 100 - 25 + sigma && ind1val2 < 100 - 25 - sigma;

   Signal signal = CreateEntrySignal(29, ind0long && ind1long, ind0short && ind1short, 0, 63, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_029()
  {
   double ind2val1 = iMFI(NULL,0,43,1);
   double ind2val2 = iMFI(NULL,0,43,2);
   bool ind2long  = ind2val1 < 72 - sigma && ind2val2 > 72 + sigma;
   bool ind2short = ind2val1 > 100 - 72 + sigma && ind2val2 < 100 - 72 - sigma;

   Signal signal = CreateExitSignal(29, ind2long, ind2short, 0, 63, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_030()
  {
   double ind0val1 = iMFI(NULL,0,9,1);
   double ind0val2 = iMFI(NULL,0,9,2);
   bool ind0long  = ind0val1 < 89 - sigma && ind0val2 > 89 + sigma;
   bool ind0short = ind0val1 > 100 - 89 + sigma && ind0val2 < 100 - 89 - sigma;

   Signal signal = CreateEntrySignal(30, ind0long, ind0short, 57, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_030()
  {
   double ind1val1 = iStochastic(NULL,0,3,1,1,MODE_SMA,0,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,3,1,1,MODE_SMA,0,MODE_SIGNAL,1);
   bool ind1long  = ind1val1 < ind1val2 - sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma;

   Signal signal = CreateExitSignal(30, ind1long, ind1short, 57, 0, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_031()
  {
   double ind0val1 = iMA(NULL,0,10,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,15,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,10,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,15,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   double ind1val1 = iRVI(NULL,0,32,MODE_MAIN,1);
   double ind1val2 = iRVI(NULL,0,32,MODE_MAIN,2);
   double ind1val3 = iRVI(NULL,0,32,MODE_MAIN,3);
   bool ind1long  = ind1val1 > ind1val2 + sigma && ind1val2 < ind1val3 - sigma;
   bool ind1short = ind1val1 < ind1val2 - sigma && ind1val2 > ind1val3 + sigma;
   double ind2val1 = iAO(NULL,0,1);
   double ind2val2 = iAO(NULL,0,2);
   bool ind2long  = ind2val1 > ind2val2 + sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma;

   Signal signal = CreateEntrySignal(31, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 31, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_031()
  {
   double ind3val1 = iDeMarker(NULL,0,15,1);
   double ind3val2 = iDeMarker(NULL,0,15,2);
   bool ind3long  = ind3val1 > 0.90 + sigma && ind3val2 < 0.90 - sigma;
   bool ind3short = ind3val1 < 1 - 0.90 - sigma && ind3val2 > 1 - 0.90 + sigma;

   Signal signal = CreateExitSignal(31, ind3long, ind3short, 0, 31, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_032()
  {
   double ind0val1 = iStochastic(NULL,0,6,4,3,MODE_SMA,0,MODE_MAIN,1);
   double ind0val2 = iStochastic(NULL,0,6,4,3,MODE_SMA,0,MODE_SIGNAL,1);
   double ind0val3 = iStochastic(NULL,0,6,4,3,MODE_SMA,0,MODE_MAIN,2);
   double ind0val4 = iStochastic(NULL,0,6,4,3,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind0long  = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   bool ind0short = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   double ind1val1 = iStochastic(NULL,0,3,2,2,MODE_SMA,0,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,3,2,2,MODE_SMA,0,MODE_SIGNAL,1);
   double ind1val3 = iStochastic(NULL,0,3,2,2,MODE_SMA,0,MODE_MAIN,2);
   double ind1val4 = iStochastic(NULL,0,3,2,2,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind1long  = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;
   bool ind1short = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   double ind2val1 = iCCI(NULL,0,33,PRICE_TYPICAL,1);
   double ind2val2 = iCCI(NULL,0,33,PRICE_TYPICAL,2);
   bool ind2long  = ind2val1 < ind2val2 - sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma;

   Signal signal = CreateEntrySignal(32, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_032()
  {
   double ind3val1 = iRVI(NULL,0,36,MODE_MAIN,1);
   double ind3val2 = iRVI(NULL,0,36,MODE_MAIN,2);
   bool ind3long  = ind3val1 < 0.00 - sigma && ind3val2 > 0.00 + sigma;
   bool ind3short = ind3val1 > -1*0.00 + sigma && ind3val2 < -1*0.00 - sigma;

   Signal signal = CreateExitSignal(32, ind3long, ind3short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_033()
  {
   double ind0val1 = iAO(NULL,0,1);
   double ind0val2 = iAO(NULL,0,2);
   bool ind0long  = ind0val1 < 0.0000 - sigma && ind0val2 > 0.0000 + sigma;
   bool ind0short = ind0val1 > -1*0.0000 + sigma && ind0val2 < -1*0.0000 - sigma;
   double ind1val1 = iStdDev(NULL,0,5,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iStdDev(NULL,0,5,0,MODE_SMA,PRICE_CLOSE,2);
   double ind1val3 = iStdDev(NULL,0,5,0,MODE_SMA,PRICE_CLOSE,3);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val2 > ind1val3 + sigma;
   bool ind1short = ind1long;
   double ind2val1 = iMA(NULL,0,20,0,MODE_SMA,PRICE_CLOSE,1);
   double ind2val2 = iMA(NULL,0,23,0,MODE_SMA,PRICE_CLOSE,1);
   bool ind2long  = ind2val1 > ind2val2 + sigma;
   bool ind2short = ind2val1 < ind2val2 - sigma;

   Signal signal = CreateEntrySignal(33, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_033()
  {
   double ind3val1 = iMFI(NULL,0,28,1);
   double ind3val2 = iMFI(NULL,0,28,2);
   bool ind3long  = ind3val1 > 65 + sigma && ind3val2 < 65 - sigma;
   bool ind3short = ind3val1 < 100 - 65 - sigma && ind3val2 > 100 - 65 + sigma;

   Signal signal = CreateExitSignal(33, ind3long, ind3short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_034()
  {

   bool ind0long  = false;
   bool ind0short = false;

   {
      const double closeOpen = MathAbs(Close(1) - Open(1));
      const bool isPinBar = closeOpen < 9 * 0.01 * (High(1) - Low(1));
      const bool upWick = High(1) - MathMax(Close(1), Open(1)) > 42 * 0.01 * closeOpen;
      const bool downWick = MathMin(Close(1), Open(1)) - Low(1) > 42 * 0.01 * closeOpen;
      const bool isBearishPinBar = isPinBar && upWick && !downWick;
      const bool isBullishPinBar = isPinBar && downWick && !upWick;

      ind0long  = isBullishPinBar;
      ind0short = isBearishPinBar;
   }

   Signal signal = CreateEntrySignal(34, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_034()
  {
   double ind1val1 = iRSI(NULL,0,12,PRICE_CLOSE,1);
   double ind1val2 = iRSI(NULL,0,12,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 > 20 + sigma && ind1val2 < 20 - sigma;
   bool ind1short = ind1val1 < 100 - 20 - sigma && ind1val2 > 100 - 20 + sigma;

   Signal signal = CreateExitSignal(34, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_035()
  {
   double ind0val1 = iMA(NULL,0,39,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,35,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,39,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,35,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   bool ind0short = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;

   Signal signal = CreateEntrySignal(35, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_035()
  {
   double ind1upBand1 = iBands(NULL,0,20,2.84,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,20,2.84,0,PRICE_CLOSE,MODE_LOWER,1);
   bool ind1long  = Open(0) > ind1upBand1 + sigma;
   bool ind1short = Open(0) < ind1dnBand1 - sigma;

   Signal signal = CreateExitSignal(35, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_036()
  {
   double ind0val1 = iRSI(NULL,0,18,PRICE_CLOSE,1);
   double ind0val2 = iRSI(NULL,0,18,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 30 - sigma && ind0val2 > 30 + sigma;
   bool ind0short = ind0val1 > 100 - 30 + sigma && ind0val2 < 100 - 30 - sigma;

   Signal signal = CreateEntrySignal(36, ind0long, ind0short, 62, 64, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_036()
  {
   double ind1upBand1 = iBands(NULL,0,23,2.23,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,23,2.23,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind1upBand2 = iBands(NULL,0,23,2.23,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind1dnBand2 = iBands(NULL,0,23,2.23,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind1long  = Open(0) > ind1upBand1 + sigma && Open(1) < ind1upBand2 - sigma;
   bool ind1short = Open(0) < ind1dnBand1 - sigma && Open(1) > ind1dnBand2 + sigma;

   Signal signal = CreateExitSignal(36, ind1long, ind1short, 62, 64, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_037()
  {
   double ind0val1 = iAC(NULL,0,1);
   double ind0val2 = iAC(NULL,0,2);
   bool ind0long  = ind0val1 > 0.0000 + sigma && ind0val2 < 0.0000 - sigma;
   bool ind0short = ind0val1 < -1*0.0000 - sigma && ind0val2 > -1*0.0000 + sigma;

   Signal signal = CreateEntrySignal(37, ind0long, ind0short, 59, 86, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_037()
  {
   double ind1upBand1 = iEnvelopes(NULL,0,33,MODE_SMA,0,PRICE_CLOSE,0.38,MODE_UPPER,1);
   double ind1dnBand1 = iEnvelopes(NULL,0,33,MODE_SMA,0,PRICE_CLOSE,0.38,MODE_LOWER,1);
   double ind1upBand2 = iEnvelopes(NULL,0,33,MODE_SMA,0,PRICE_CLOSE,0.38,MODE_UPPER,2);
   double ind1dnBand2 = iEnvelopes(NULL,0,33,MODE_SMA,0,PRICE_CLOSE,0.38,MODE_LOWER,2);
   bool ind1long  = Open(0) < ind1upBand1 - sigma && Open(1) > ind1upBand2 + sigma;
   bool ind1short = Open(0) > ind1dnBand1 + sigma && Open(1) < ind1dnBand2 - sigma;

   Signal signal = CreateExitSignal(37, ind1long, ind1short, 59, 86, true);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_038()
  {

   bool ind0long  = false;
   bool ind0short = false;

   {
      const double closeOpen = MathAbs(Close(1) - Open(1));
      const bool isPinBar = closeOpen < 8 * 0.01 * (High(1) - Low(1));
      const bool upWick = High(1) - MathMax(Close(1), Open(1)) > 15 * 0.01 * closeOpen;
      const bool downWick = MathMin(Close(1), Open(1)) - Low(1) > 15 * 0.01 * closeOpen;
      const bool isBearishPinBar = isPinBar && upWick && !downWick;
      const bool isBullishPinBar = isPinBar && downWick && !upWick;

      ind0long  = isBullishPinBar;
      ind0short = isBearishPinBar;
   }

   Signal signal = CreateEntrySignal(38, ind0long, ind0short, 86, 92, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_038()
  {
   double ind1upBand1 = iBands(NULL,0,31,3.72,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,31,3.72,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind1upBand2 = iBands(NULL,0,31,3.72,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind1dnBand2 = iBands(NULL,0,31,3.72,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind1long  = Open(0) > ind1dnBand1 + sigma && Open(1) < ind1dnBand2 - sigma;
   bool ind1short = Open(0) < ind1upBand1 - sigma && Open(1) > ind1upBand2 + sigma;

   Signal signal = CreateExitSignal(38, ind1long, ind1short, 86, 92, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_039()
  {
   double ind0upBand1 = iEnvelopes(NULL,0,22,MODE_SMA,0,PRICE_CLOSE,0.09,MODE_UPPER,1);
   double ind0dnBand1 = iEnvelopes(NULL,0,22,MODE_SMA,0,PRICE_CLOSE,0.09,MODE_LOWER,1);
   double ind0upBand2 = iEnvelopes(NULL,0,22,MODE_SMA,0,PRICE_CLOSE,0.09,MODE_UPPER,2);
   double ind0dnBand2 = iEnvelopes(NULL,0,22,MODE_SMA,0,PRICE_CLOSE,0.09,MODE_LOWER,2);
   bool ind0long  = Open(0) < ind0dnBand1 - sigma && Open(1) > ind0dnBand2 + sigma;
   bool ind0short = Open(0) > ind0upBand1 + sigma && Open(1) < ind0upBand2 - sigma;
   double ind1val1 = iMA(NULL,0,13,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,13,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = Open(0) < ind1val1 - sigma && Open(1) > ind1val2 + sigma;
   bool ind1short = Open(0) > ind1val1 + sigma && Open(1) < ind1val2 - sigma;

   Signal signal = CreateEntrySignal(39, ind0long && ind1long, ind0short && ind1short, 14, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_039()
  {
   double ind2val1 = iADX(NULL,0,48,PRICE_CLOSE,1,1) - iADX(NULL,0,48,PRICE_CLOSE,2,1);
   double ind2val2 = iADX(NULL,0,48,PRICE_CLOSE,1,2) - iADX(NULL,0,48,PRICE_CLOSE,2,2);
   bool ind2long  = ind2val1 > 0 + sigma && ind2val2 < 0 - sigma;
   bool ind2short = ind2val1 < 0 - sigma && ind2val2 > 0 + sigma;

   Signal signal = CreateExitSignal(39, ind2long, ind2short, 14, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_040()
  {
   double ind0val1 = iForce(NULL,0,13,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iForce(NULL,0,13,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 > 0 + sigma && ind0val2 < 0 - sigma;
   bool ind0short = ind0val1 < 0 - sigma && ind0val2 > 0 + sigma;

   Signal signal = CreateEntrySignal(40, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_040()
  {
   double ind1upBand1 = iBands(NULL,0,21,3.00,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,21,3.00,0,PRICE_CLOSE,MODE_LOWER,1);
   double ind1upBand2 = iBands(NULL,0,21,3.00,0,PRICE_CLOSE,MODE_UPPER,2);
   double ind1dnBand2 = iBands(NULL,0,21,3.00,0,PRICE_CLOSE,MODE_LOWER,2);
   bool ind1long  = Open(0) > ind1upBand1 + sigma && Open(1) < ind1upBand2 - sigma;
   bool ind1short = Open(0) < ind1dnBand1 - sigma && Open(1) > ind1dnBand2 + sigma;

   Signal signal = CreateExitSignal(40, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_041()
  {
   double ind0val1 = iCCI(NULL,0,40,PRICE_TYPICAL,1);
   double ind0val2 = iCCI(NULL,0,40,PRICE_TYPICAL,2);
   bool ind0long  = ind0val1 < 24 - sigma && ind0val2 > 24 + sigma;
   bool ind0short = ind0val1 > -1*24 + sigma && ind0val2 < -1*24 - sigma;
   double ind1val1 = iADX(NULL,0,27,PRICE_CLOSE,1,1) - iADX(NULL,0,27,PRICE_CLOSE,2,1);
   double ind1val2 = iADX(NULL,0,27,PRICE_CLOSE,1,2) - iADX(NULL,0,27,PRICE_CLOSE,2,2);
   bool ind1long  = ind1val1 < 0 - sigma && ind1val2 > 0 + sigma;
   bool ind1short = ind1val1 > 0 + sigma && ind1val2 < 0 - sigma;
   double ind2val1 = iMA(NULL,0,38,0,MODE_SMA,PRICE_CLOSE,1);
   double ind2val2 = iMA(NULL,0,38,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind2long  = ind2val1 < ind2val2 - sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma;

   Signal signal = CreateEntrySignal(41, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 0, 31, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_041()
  {
   double ind3val1 = iRSI(NULL,0,33,PRICE_CLOSE,1);
   double ind3val2 = iRSI(NULL,0,33,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > 61 + sigma && ind3val2 < 61 - sigma;
   bool ind3short = ind3val1 < 100 - 61 - sigma && ind3val2 > 100 - 61 + sigma;

   Signal signal = CreateExitSignal(41, ind3long, ind3short, 0, 31, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_042()
  {
   double ind0val1 = iMFI(NULL,0,5,1);
   double ind0val2 = iMFI(NULL,0,5,2);
   bool ind0long  = ind0val1 < 72 - sigma && ind0val2 > 72 + sigma;
   bool ind0short = ind0val1 > 100 - 72 + sigma && ind0val2 < 100 - 72 - sigma;

   Signal signal = CreateEntrySignal(42, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_042()
  {
   double ind1val1 = iAO(NULL,0,1);
   double ind1val2 = iAO(NULL,0,2);
   bool ind1long  = ind1val1 > -0.0012 + sigma && ind1val2 < -0.0012 - sigma;
   bool ind1short = ind1val1 < -1*-0.0012 - sigma && ind1val2 > -1*-0.0012 + sigma;

   Signal signal = CreateExitSignal(42, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_043()
  {
   double ind0upBand1 = iEnvelopes(NULL,0,28,MODE_SMA,0,PRICE_CLOSE,0.22,MODE_UPPER,1);
   double ind0dnBand1 = iEnvelopes(NULL,0,28,MODE_SMA,0,PRICE_CLOSE,0.22,MODE_LOWER,1);
   double ind0upBand2 = iEnvelopes(NULL,0,28,MODE_SMA,0,PRICE_CLOSE,0.22,MODE_UPPER,2);
   double ind0dnBand2 = iEnvelopes(NULL,0,28,MODE_SMA,0,PRICE_CLOSE,0.22,MODE_LOWER,2);
   bool ind0long  = Open(0) < ind0dnBand1 - sigma && Open(1) > ind0dnBand2 + sigma;
   bool ind0short = Open(0) > ind0upBand1 + sigma && Open(1) < ind0upBand2 - sigma;

   Signal signal = CreateEntrySignal(43, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_043()
  {
   double ind1val1 = iMomentum(NULL,0,68,PRICE_CLOSE,1);
   double ind1val2 = iMomentum(NULL,0,68,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 > ind1val2 + sigma;
   bool ind1short = ind1val1 < ind1val2 - sigma;

   Signal signal = CreateExitSignal(43, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_044()
  {
   double ind0val1 = iAO(NULL,0,1);
   double ind0val2 = iAO(NULL,0,2);
   bool ind0long  = ind0val1 < 0.0000 - sigma && ind0val2 > 0.0000 + sigma;
   bool ind0short = ind0val1 > -1*0.0000 + sigma && ind0val2 < -1*0.0000 - sigma;

   Signal signal = CreateEntrySignal(44, ind0long, ind0short, 0, 23, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_044()
  {
   double ind1val1 = iStochastic(NULL,0,8,1,8,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,8,1,8,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind1long  = ind1val1 > 92 + sigma && ind1val2 < 92 - sigma;
   bool ind1short = ind1val1 < 100 - 92 - sigma && ind1val2 > 100 - 92 + sigma;

   Signal signal = CreateExitSignal(44, ind1long, ind1short, 0, 23, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_045()
  {

   bool ind0long  = false;
   bool ind0short = false;

   {
      const double closeOpen = MathAbs(Close(1) - Open(1));
      const bool isPinBar = closeOpen < 6 * 0.01 * (High(1) - Low(1));
      const bool upWick = High(1) - MathMax(Close(1), Open(1)) > 17 * 0.01 * closeOpen;
      const bool downWick = MathMin(Close(1), Open(1)) - Low(1) > 17 * 0.01 * closeOpen;
      const bool isBearishPinBar = isPinBar && upWick && !downWick;
      const bool isBullishPinBar = isPinBar && downWick && !upWick;

      ind0long  = isBullishPinBar;
      ind0short = isBearishPinBar;
   }

   Signal signal = CreateEntrySignal(45, ind0long, ind0short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_045()
  {
   double ind1upBand1 = iEnvelopes(NULL,0,27,MODE_SMA,0,PRICE_CLOSE,0.12,MODE_UPPER,1);
   double ind1dnBand1 = iEnvelopes(NULL,0,27,MODE_SMA,0,PRICE_CLOSE,0.12,MODE_LOWER,1);
   bool ind1long  = Open(0) < ind1dnBand1 - sigma;
   bool ind1short = Open(0) > ind1upBand1 + sigma;

   Signal signal = CreateExitSignal(45, ind1long, ind1short, 0, 0, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
ENUM_INIT_RETCODE ValidateInit()
  {
   return (INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
/*STRATEGY MARKET ICMarkets-Live16; EURUSD; M5 */
